@@ -1,16 +1,27 @@
 from __future__ import annotations
 
 import argparse
+from typing import TYPE_CHECKING
 
 import cfonts
-
+import tomlkit
 from packaging.specifiers import SpecifierSet
+
+from pdm.exceptions import ProjectError
+from pdm.formats import FORMATS
 from pdm.iostream import stream
 from pdm.models.candidates import identify
 from pdm.models.environment import WorkingSet
 from pdm.models.requirements import Requirement, strip_extras
+from pdm.models.specifiers import bump_version, get_specifier
+from pdm.project import Project
 from pdm.resolver.structs import DirectedGraph
 
+if TYPE_CHECKING:
+    from pathlib import Path
+    from typing import Dict, Iterable, Tuple
+    from pdm.models.candidates import Candidate
+
 
 class PdmFormatter(argparse.HelpFormatter):
     def _format_action(self, action):
@@ -240,3 +251,100 @@ def format_dependency_graph(graph: DirectedGraph) -> str:
     for package in graph.iter_children(None):
         content.append(format_package(graph, package, prefix="", visited=set()))
     return "".join(content).strip()
+
+
+def format_lockfile(mapping, fetched_dependencies, summary_collection):
+    """Format lock file from a dict of resolved candidates, a mapping of dependencies
+    and a collection of package summaries.
+    """
+    packages = tomlkit.aot()
+    metadata = tomlkit.table()
+    for k, v in sorted(mapping.items()):
+        base = tomlkit.table()
+        base.update(v.as_lockfile_entry())
+        base.add("summary", summary_collection[strip_extras(k)[0]])
+        deps = tomlkit.table()
+        for r in fetched_dependencies[k].values():
+            name, req = r.as_req_dict()
+            if getattr(req, "items", None) is not None:
+                inline = tomlkit.inline_table()
+                inline.update(req)
+                deps.add(name, inline)
+            else:
+                deps.add(name, req)
+        if len(deps) > 0:
+            base.add("dependencies", deps)
+        packages.append(base)
+        if v.hashes:
+            key = f"{k} {v.version}"
+            array = tomlkit.array()
+            array.multiline(True)
+            for filename, hash_value in v.hashes.items():
+                inline = tomlkit.inline_table()
+                inline.update({"file": filename, "hash": hash_value})
+                array.append(inline)
+            if array:
+                metadata.add(key, array)
+    doc = tomlkit.document()
+    doc.update({"package": packages, "metadata": metadata})
+    return doc
+
+
+def save_version_specifiers(
+    requirements: Dict[str, Requirement],
+    resolved: Dict[str, Candidate],
+    save_strategy: str,
+) -> None:
+    """Rewrite the version specifiers according to the resolved result and save strategy
+
+    :param requirements: the requirements to be updated
+    :param resolved: the resolved mapping
+    :param save_strategy: compatible/wildcard/exact
+    """
+    for name, r in requirements.items():
+        if r.is_named and not r.specifier:
+            if save_strategy == "exact":
+                r.specifier = get_specifier(f"=={resolved[name].version}")
+            elif save_strategy == "compatible":
+                version = str(resolved[name].version)
+                next_major_version = ".".join(
+                    map(str, bump_version(tuple(version.split(".")), 0))
+                )
+                r.specifier = get_specifier(f">={version},<{next_major_version}")
+
+
+def check_project_file(project: Project) -> None:
+    """Check the existence of the project file and throws an error on failure."""
+    if not project.tool_settings:
+        raise ProjectError(
+            "The pyproject.toml has not been initialized yet. You can do this "
+            "by running {}.".format(stream.green("'pdm init'"))
+        )
+
+
+def format_toml(pdm_settings):
+    """Ensure the dependencies are inline tables"""
+    for section in pdm_settings:
+        if not section.endswith("dependencies"):
+            continue
+        for name in pdm_settings[section]:
+            if getattr(pdm_settings[section][name], "items", None):
+                table = tomlkit.inline_table()
+                table.update(pdm_settings[section][name])
+                pdm_settings[section][name] = table
+
+
+def find_importable_files(project: Project) -> Iterable[Tuple[str, Path]]:
+    """Find all possible files that can be imported"""
+    for filename in (
+        "Pipfile",
+        "pyproject.toml",
+        "requirements.in",
+        "requirements.txt",
+    ):
+        project_file = project.root / filename
+        if not project_file.exists():
+            continue
+        for key, module in FORMATS.items():
+            if module.check_fingerprint(project, project_file.as_posix()):
+                yield key, project_file
