@@ -15,12 +15,9 @@ from pip._vendor import packaging, pkg_resources
 from pip_shims import shims
 from pythonfinder import Finder
 from pythonfinder.environment import PYENV_INSTALLED, PYENV_ROOT
-from vistir.contextmanagers import temp_environ
-from vistir.path import normalize_path
 
 from pdm.exceptions import NoPythonVersion
 from pdm.iostream import stream
-from pdm.models.specifiers import PySpecSet
 from pdm.utils import (
     allow_all_wheels,
     cached_property,
@@ -31,6 +28,7 @@ from pdm.utils import (
     get_python_version,
     get_sys_config_paths,
     get_venv_python,
+    temp_environ,
 )
 
 if TYPE_CHECKING:
@@ -84,6 +82,7 @@ class Environment:
         """
         self.python_requires = project.python_requires
         self.project = project
+        self._wheel_ensured = False
 
     @cached_property
     def python_executable(self) -> str:
@@ -181,8 +180,8 @@ class Environment:
 
     def is_local(self, path) -> bool:
         """PEP 582 version of ``is_local()`` function."""
-        return normalize_path(path).startswith(
-            normalize_path(self.packages_path.as_posix())
+        return misc.normalize_path(path).startswith(
+            misc.normalize_path(self.packages_path.as_posix())
         )
 
     def evaluate_marker(self, text: str, extra=None) -> bool:
@@ -309,7 +308,11 @@ class Environment:
             # Now all source is prepared, build it.
             if ireq.link.is_wheel:
                 return (self.project.cache("wheels") / ireq.link.filename).as_posix()
-            builder_class = EditableBuilder if ireq.editable else WheelBuilder
+            if ireq.editable:
+                builder_class = EditableBuilder
+            else:
+                builder_class = WheelBuilder
+                self.ensure_wheel_package()
             kwargs["finder"] = finder
             with builder_class(ireq) as builder, self.activate():
                 return builder.build(**kwargs)
@@ -337,18 +340,25 @@ class Environment:
         # Fallback to use shutil.which to find the executable
         return shutil.which(command, path=os.getenv("PATH"))
 
+    def ensure_wheel_package(self) -> None:
+        """Ensure wheel package is available and install if it isn't."""
+        from pdm.installers import Installer
+        from pdm.models.requirements import parse_requirement
+        from pdm.models.candidates import Candidate
+
+        if self._wheel_ensured or "wheel" in self.get_working_set():
+            return
+        req = parse_requirement("wheel")
+        candidate = Candidate(req, self, "wheel")
+        Installer(self).install(candidate)
+        self._wheel_ensured = True
+
 
 class GlobalEnvironment(Environment):
     """Global environment"""
 
     is_global = True
 
-    def __init__(self, project: Project) -> None:
-        super().__init__(project)
-        self.python_requires = PySpecSet(
-            "==" + get_python_version(self.python_executable, True)
-        )
-
     def get_paths(self) -> Dict[str, str]:
         paths = get_sys_config_paths(self.python_executable)
         paths["prefix"] = paths["data"]
@@ -356,8 +366,8 @@ class GlobalEnvironment(Environment):
         return paths
 
     def is_local(self, path) -> bool:
-        return normalize_path(path).startswith(
-            normalize_path(self.get_paths()["prefix"])
+        return misc.normalize_path(path).startswith(
+            misc.normalize_path(self.get_paths()["prefix"])
         )
 
     @property
